#include <STDIO.H>
#include "FSEDCONF.H"

read_block()
{
  if ((fd = open("FSED.COM",2)) == ERROR)
  {
    p("\nFSEDCONF requires copy of FSED.COM")
    p("to be present in the current logged directory.")
    p("Please copy them to this directory now, then run FSEDCONF again.\n")
    exit();
  }
  if (read(fd, secbuf, 1) != 1)
  {
    p("\nDisk error on reading FSED.COM.\n")
    exit();
  }
}

write_block()
{
  seek(fd, 0, 0);
  if (write(fd, secbuf, 1) != 1)
  {
    p("\nError on writing FSED.COM.")
    p("Please place fresh copy of FSED.COM")
    p("in current directory and run FSEDCONF again.")
    p("\nSorry, but I don't know why this happend.\n")
    exit();
  }
  close(fd);
  p("\nFSED.COM successfully updated.\n")
}

int ask(txt)
  char *txt;
{
  char c;

  if (txt)
    p(txt)

  while (1)
  {
    column = 0; /* prevent spurious newlines */
    if ((c = toupper(getch())) == 'Y')
      return TRUE;
    else if (c == 'N')
      return FALSE;
    p("Please answer 'y' or 'n'.")
  }
}

int getch() /* get a char of text */
{
  int c;
  if ((c = getchar()) == -1)
    longjmp(jbuf);
  if (c == 13 || c == 10)
  {
    for (column = 0; column < indent; column++)
      putchar(' ');
  }
  else
    p("\n")
  return c;
}

pressKey()
{
  p("- Press key to continue -")
  getch();
}

prnt(str) /* print given text, automatically filling to length of line */
  char *str;
{
  char c;
  int i;

  while (c = *str++)
  {
    if (c != '\n' && (wdlen(str) + column) < maxcol)
    {
      putchar(c);
      column++;
      if (!*str)
      {
	putchar(' ');
	column++;
      }
      continue;
    }

    putchar('\n');

    if (indent > 0)
    {
      for (i = 0; i < indent; i++)
	putchar(' ');
      column = indent;
    }
    else
      column = 0;

    if (!isspace(c))
    {
      putchar(c);
      column++;
    }
  }
}

int wdlen(txt) /* return length of text word */
  char *txt;
{
  int i;
  i = 0;
  while (*txt) {
    if (isspace(*txt))
      break;
    txt++;
    i++;
  }
  return i;
}

char* prep_code(c, buf)
  char c;
  char *buf;
{
  if (c == 0xFF)
    strcpy(buf, "Undefined");
  else
  {
    sprintf(buf, "%02x ", c);
    if (c == 0x7F)
      strcat(buf, "DEL");
    else if (c == 0)
      strcat(buf, "NUL");
    else if (c < 32)
      sprintf(buf + 3, "Ctrl-%c", c + '@');
/*
    else if (c >= 0x80 && c < 0xA0)
      sprintf(buf + 3, "Upper Ctrl-%c", c - 0x80 + '@');
    else
      sprintf(buf + 3, "%c", c);
*/
  }
  return buf;
}

char* left_trim(in_out)
  char *in_out;
{
  char *w, *q;

  if (isspace(*in_out))
  {
    w = in_out + 1;
    while (isspace(*w))
      w++;

    q = in_out;
    while (*w)
      *q++ = *w++;
    *q = 0;
  }
  return in_out;
}

char* right_trim(in_out)
  char *in_out;
{
  char *q;

  q = &in_out[strlen(in_out)];
  while (q > in_out)
  {
    q--;
    if (isspace(*q))
      *q = 0;
    else
      break;
  }
  return in_out;
}

char* trim(in_out)
  char *in_out;
{
  return left_trim(right_trim(in_out));
}

int readline(s, max)
  char *s;
  int max;
{
  char c, *q;

  q = s;
  while (1)
  {
    c = bios(3, 0);
    if (c == 26) /* Ctrl-Z */
      longjmp(jbuf);
    if (c == 13) /* enter */
      break;
    else if (c == 8) /* backspace */
    {
      if (q - s == 0)
	continue;
      bios(4, 8);
      bios(4, ' ');
      bios(4, 8);
      q--;
    }
    else if (q - s == max)
      continue;
    else if (c >= 32 && c <= 126)
    {
      *q++ = c;
      bios(4, c);
    }
  }
  *q = 0;

  return q - s;
}

int isxdigit(c)
  char c;
{
  return (c >= '0' && c <= '9')
	  || (c >= 'A' && c <= 'F');
}

int parseValue(ps, hex, max)
  char **ps;
  int hex, max;
{
  char *q, c;
  int val, base;

  base = (hex) ? 16 : 10;
  q = *ps;
  val = 0;
  while (1)
  {
    c = toupper(*q);
    if ((hex && !isxdigit(c)) || (!hex && !isdigit(c)))
      break;
    c -= '0';
    if (c > 9)
      c -= 7;
    val = val * base + c;
    q++;
  }
  if (val <= max && isspace(c))
    q++;
  else if (c)
    val = -1;
  if (val >= 0)
    *ps = q;
  return val;
}

int readCodes(seq, cnt, pos, mand)
  char *seq;
  int cnt, pos, mand;
{
  char codes[12], *q;
  int i, cl, rw, v;

  setmem(codes, cnt, 0);
  readline(strbuf, 38);
  trim(strbuf);
  i = strlen(strbuf);
  if (!i)
  {
    p("\tUnchanged\n")
    return 0;
  }

  cl = 0;
  rw = 0;
  q = strbuf;
  for (i = 0; i < cnt; i++)
  {
    trim(q);
    if (!*q)
      break;

    v = parseValue(&q, 1, 0xFF);
    if (v < 0)
    {
      if (pos)
      {
	if (cmpRC(&q, "COL"))
	{
	  if (!cl)
	  {
	    cl = i;
	    continue;
	  }
	  p("\tDuplicate 'col'!")
	  return -1;
	}
	else if (cmpRC(&q, "ROW"))
	{
	  if (!rw)
	  {
	    rw = i;
	    continue;
	  }
	  p("\tDuplicate 'row'!")
	  return -1;
	}
      }
      p("\tBad entry!")
      return -1;
    }
    else if (!i && ((mand && !v) || v > 0x1F))
    {
      if (mand)
	p("\tMandatory!")
      else
	p("\tBad entry!")
      return -1;
    }
    else
      codes[i] = v;
  }

  trim(q);
  if (i == cnt && *q)
    return -1;

  if (pos)
  {
    if (!rw || !cl)
    {
      p("\tMissing 'col' and/or 'row'!")
      return -1;
    }
    secbuf[38] = rw;
    secbuf[39] = cl;
  }

  movmem(codes, seq, cnt);
  seq[i] = 0;
  return 1;
}

int cmpRC(pb, s)
  char **pb;
  char *s;
{
  char *q;

  q = *pb;
  if (toupper(q[0]) == s[0]
      && toupper(q[1]) == s[1]
      && toupper(q[2]) == s[2]
      && (q[3] == ' ' || !q[3]))
  {
    if (q[3])
      *pb = q + 4;
    else
      *pb = q + 3;
    return 1;
  }
  return 0;
}

